@@ -15,7 +15,7 @@ if [[ ${PREFIX_ARG} ]]; then
   rm -rf "${PREFIX_ARG}"/*
 fi
 
-# *never* install with ASAN enabled
-meson configure -Db_sanitize=none "${CURRENT_BUILD_DIR:-build}"
+# Ensure code coverage and ASAN are disabled before installing
+meson configure -Db_coverage=false -Db_sanitize=none "${CURRENT_BUILD_DIR:-build}"
 
 DESTDIR="${DESTDIR:-/}" ninja -C "${CURRENT_BUILD_DIR:-build}" -v install
