@@ -2,8 +2,19 @@
 
 #include <pbbam/DataSet.h>
 
+#include <pbbam/internal/DataSetBaseTypes.h>
+#include "DataSetIO.h"
+#include "DataSetUtils.h"
+#include "FileUtils.h"
+#include "TimeUtils.h"
+
+#include <boost/algorithm/string.hpp>
+#include <boost/icl/interval_set.hpp>
+#include <boost/lexical_cast.hpp>
+
 #include <algorithm>
 #include <map>
+#include <optional>
 #include <ostream>
 #include <sstream>
 #include <stdexcept>
@@ -11,18 +22,6 @@
 #include <unordered_map>
 #include <vector>
 
-#include <boost/algorithm/string.hpp>
-#include <boost/icl/interval_set.hpp>
-#include <boost/lexical_cast.hpp>
-#include <boost/optional.hpp>
-
-#include <pbbam/internal/DataSetBaseTypes.h>
-
-#include "DataSetIO.h"
-#include "DataSetUtils.h"
-#include "FileUtils.h"
-#include "TimeUtils.h"
-
 namespace PacBio {
 namespace BAM {
 namespace {
@@ -37,8 +36,9 @@ void GetAllFiles(const ExternalResources& resources, std::vector<std::string>* r
         result->push_back(resource.ResourceId());
 
         // store any child indices
-        for (const auto& idx : resource.FileIndices())
+        for (const auto& idx : resource.FileIndices()) {
             result->push_back(idx.ResourceId());
+        }
 
         // recurse into any other child resources
         GetAllFiles(resource.ExternalResources(), result);
@@ -109,12 +109,15 @@ DataSet::DataSet(const std::string& filename) : d_(DataSetIO::FromUri(filename))
     if (boost::algorithm::iends_with(filename, ".fofn") ||
         boost::algorithm::iends_with(filename, ".bam") ||
         boost::algorithm::iends_with(filename, ".fasta") ||
+        boost::algorithm::iends_with(filename, ".fsa") ||
         boost::algorithm::iends_with(filename, ".fa")) {
         d_->Path(FileUtils::CurrentWorkingDirectory());
     }
 
     else {
-        if (boost::algorithm::iends_with(filename, ".xml")) d_->FromInputXml(true);
+        if (boost::algorithm::iends_with(filename, ".xml")) {
+            d_->FromInputXml(true);
+        }
         d_->Path(FileUtils::DirectoryName(filename));
     }
 }
@@ -137,7 +140,9 @@ DataSet::DataSet(const DataSet& other)
 
 DataSet& DataSet::operator=(const DataSet& other)
 {
-    if (this != &other) *this = DataSet{other};
+    if (this != &other) {
+        *this = DataSet{other};
+    }
     return *this;
 }
 
@@ -289,9 +294,9 @@ std::vector<GenomicInterval> DataSet::GenomicIntervals() const
             const int32_t refLength = boost::lexical_cast<int32_t>(header.SequenceLength(i));
 
             const auto it = contigLengths.find(refName);
-            if (it == contigLengths.cend())
+            if (it == contigLengths.cend()) {
                 contigLengths.emplace(refName, refLength);
-            else if (it->second != refLength) {
+            } else if (it->second != refLength) {
                 throw std::runtime_error{"[pbbam] dataset ERROR: " + refName +
                                          " occurs twice with different lengths ('" +
                                          std::to_string(it->second) + "' and '" +
@@ -310,7 +315,7 @@ std::vector<GenomicInterval> DataSet::GenomicIntervals() const
 
     for (const auto& xmlFilter : Filters()) {
         ++numFilters;
-        boost::optional<std::string> contigName;
+        std::optional<std::string> contigName;
 
         intT intersectedInterval{intInterval{0, std::numeric_limits<int32_t>::max()}};
 
@@ -324,40 +329,46 @@ std::vector<GenomicInterval> DataSet::GenomicIntervals() const
                     contigName = XmlValue;
 
                     const auto it = contigLengths.find(XmlValue);
-                    if (it == contigLengths.cend())
+                    if (it == contigLengths.cend()) {
                         throw std::runtime_error{"[pbbam] dataset ERROR: could not find contig '" +
                                                  XmlValue + "' in BAM files"};
-                    else
+                    } else {
                         intersectedInterval &= intInterval(0, it->second);
-                } else
+                    }
+                } else {
                     throw std::runtime_error{
                         "[pbbam] dataset ERROR: '" + XmlOperator +
                         "' is an unrecognized property operator, only '=' is recognized"};
+                }
             } else if ("tstart" == XmlName) {
-                if ((XmlOperator != "<") && (XmlOperator != "<="))
+                if ((XmlOperator != "<") && (XmlOperator != "<=")) {
                     throw std::runtime_error{
                         "[pbbam] dataset ERROR: 'tstart' only supports '<' and '<=' operators"};
+                }
 
                 const int32_t end = boost::lexical_cast<int32_t>(XmlValue) + ("<=" == XmlOperator);
                 intersectedInterval &= intInterval(0, end);
             } else if ("tend" == XmlName) {
-                if ((XmlOperator != ">") && (XmlOperator != ">="))
+                if ((XmlOperator != ">") && (XmlOperator != ">=")) {
                     throw std::runtime_error{
                         "[pbbam] dataset ERROR: 'tend' only supports '>' and '>=' operators"};
+                }
 
                 const int32_t start =
                     boost::lexical_cast<int32_t>(XmlValue) - (">=" == XmlOperator);
                 intersectedInterval &= intInterval(start, std::numeric_limits<int32_t>::max());
-            } else
+            } else {
                 throw std::runtime_error{"[pbbam] dataset ERROR: '" + XmlName +
                                          "' is an unrecognized filter property name"};
+            }
         }
 
-        if (contigName)
+        if (contigName) {
             contigIntervals[contigName.value()] |= intersectedInterval;
-        else
+        } else {
             throw std::runtime_error{
                 "[pbbam] dataset ERROR: current filter does not have a valid 'rname' attribute"};
+        }
     }
 
     // extract all GenomicIntervals
@@ -368,13 +379,16 @@ std::vector<GenomicInterval> DataSet::GenomicIntervals() const
             const auto& contigName = contigs.first;
             for (const auto& i : contigs.second) {
                 // don't append empty intervals to the result
-                if (boost::icl::length(i)) result.emplace_back(contigName, i.lower(), i.upper());
+                if (boost::icl::length(i)) {
+                    result.emplace_back(contigName, i.lower(), i.upper());
+                }
             }
         }
     } else {
         // no filters, return complete list of intervals
-        for (const auto& contigs : contigLengths)
+        for (const auto& contigs : contigLengths) {
             result.emplace_back(contigs.first, 0, contigs.second);
+        }
     }
 
     return result;
@@ -488,13 +502,27 @@ std::set<std::string> DataSet::SequencingChemistries() const
 {
     std::set<std::string> result;
     for (const auto& bf : BamFiles()) {
-        if (!bf.IsPacBioBAM())
+        if (!bf.IsPacBioBAM()) {
             throw std::runtime_error{
                 "[pbbam] dataset ERROR: only PacBio BAMs are supported for fetching chemistry "
                 "info"};
+        }
         const std::vector<ReadGroupInfo> readGroups{bf.Header().ReadGroups()};
-        for (const auto& rg : readGroups)
+        for (const auto& rg : readGroups) {
             result.insert(rg.SequencingChemistry());
+        }
+    }
+    return result;
+}
+
+std::set<std::string> DataSet::Samples() const
+{
+    std::set<std::string> result;
+    for (const auto& bf : BamFiles()) {
+        const std::vector<ReadGroupInfo> readGroups{bf.Header().ReadGroups()};
+        for (const auto& rg : readGroups) {
+            result.insert(rg.Sample());
+        }
     }
     return result;
 }
@@ -509,6 +537,19 @@ DataSet& DataSet::SubDataSets(const BAM::SubDataSets& subdatasets)
     return *this;
 }
 
+const BAM::SupplementalResources& DataSet::SupplementalResources() const
+{
+    return d_->SupplementalResources();
+}
+
+BAM::SupplementalResources& DataSet::SupplementalResources() { return d_->SupplementalResources(); }
+
+DataSet& DataSet::SupplementalResources(const BAM::SupplementalResources& resources)
+{
+    d_->SupplementalResources(resources);
+    return *this;
+}
+
 const std::string& DataSet::Tags() const { return d_->Tags(); }
 
 std::string& DataSet::Tags() { return d_->Tags(); }
@@ -537,7 +578,7 @@ DataSet& DataSet::Type(const DataSet::TypeEnum type)
     return *this;
 }
 
-std::string DataSet::TypeName() const { return d_->LocalNameLabel().to_string(); }
+std::string DataSet::TypeName() const { return std::string{d_->LocalNameLabel()}; }
 
 std::string DataSet::TypeToName(const DataSet::TypeEnum& type)
 {
