@@ -2,6 +2,7 @@
 
 #include <pbbam/PbiFilterTypes.h>
 
+#include <pbbam/RecordType.h>
 #include <pbbam/StringUtilities.h>
 
 #include <boost/algorithm/string.hpp>
@@ -102,8 +103,7 @@ bool PbiIdentityFilter::Accepts(const PbiRawData& idx, const size_t row) const
 
 PbiMovieNameFilter::PbiMovieNameFilter(const std::string& movieName, const Compare::Type cmp)
     : PbiMovieNameFilter{{1, movieName}, cmp}
-{
-}
+{}
 
 PbiMovieNameFilter::PbiMovieNameFilter(const std::vector<std::string>& movieNames,
                                        const Compare::Type cmp)
@@ -132,6 +132,7 @@ PbiMovieNameFilter::PbiMovieNameFilter(const std::vector<std::string>& movieName
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeReadGroupId(movieName, "SCRAP")));
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeReadGroupId(movieName, "UNKNOWN")));
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeReadGroupId(movieName, "ZMW")));
+        candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeReadGroupId(movieName, "SEGMENT")));
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "CCS")));
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "TRANSCRIPT")));
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "POLYMERASE")));
@@ -140,6 +141,7 @@ PbiMovieNameFilter::PbiMovieNameFilter(const std::vector<std::string>& movieName
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "SCRAP")));
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "UNKNOWN")));
         candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "ZMW")));
+        candidateRgIds_.insert(ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "SEGMENT")));
         movieNames_.insert(movieName);
     }
     // clang-format on
@@ -148,7 +150,6 @@ PbiMovieNameFilter::PbiMovieNameFilter(const std::vector<std::string>& movieName
 bool PbiMovieNameFilter::Accepts(const PbiRawData& idx, const size_t row) const
 {
     const auto accepted = [this](const PbiRawData& index, const size_t i) {
-
         // straightforward lookup
         const auto& rgId = index.BasicData().rgId_.at(i);
         const auto foundAt = candidateRgIds_.find(rgId);
@@ -184,6 +185,9 @@ bool PbiMovieNameFilter::Accepts(const PbiRawData& idx, const size_t row) const
                 return false;
             };
 
+            if (tryBarcodedType("SEGMENT")) {
+                return true;
+            }
             if (tryBarcodedType("CCS")) {
                 return true;
             }
@@ -224,8 +228,7 @@ struct PbiNumSubreadsFilter::PbiNumSubreadsFilterPrivate
 {
     PbiNumSubreadsFilterPrivate(int numSubreads, const Compare::Type cmp)
         : numSubreads_{numSubreads}, cmp_{cmp}
-    {
-    }
+    {}
 
     PbiNumSubreadsFilterPrivate(const std::unique_ptr<PbiNumSubreadsFilterPrivate>& other)
     {
@@ -281,13 +284,11 @@ struct PbiNumSubreadsFilter::PbiNumSubreadsFilterPrivate
 
 PbiNumSubreadsFilter::PbiNumSubreadsFilter(int numSubreads, const Compare::Type cmp)
     : d_{std::make_unique<PbiNumSubreadsFilter::PbiNumSubreadsFilterPrivate>(numSubreads, cmp)}
-{
-}
+{}
 
 PbiNumSubreadsFilter::PbiNumSubreadsFilter(const PbiNumSubreadsFilter& other)
     : d_{std::make_unique<PbiNumSubreadsFilter::PbiNumSubreadsFilterPrivate>(other.d_)}
-{
-}
+{}
 
 PbiNumSubreadsFilter::PbiNumSubreadsFilter(PbiNumSubreadsFilter&&) noexcept = default;
 
@@ -326,12 +327,21 @@ public:
         : cmp_{cmp}
     {
         for (const auto& queryName : queryNames) {
-
             if (queryName.find("transcript/") == 0) {
                 HandleName(queryName, RecordType::TRANSCRIPT);
             } else if (queryName.find("/ccs") != std::string::npos) {
-                HandleName(queryName, RecordType::CCS);
+
+                // normal CCS
+                if (queryName.ends_with("ccs") || queryName.ends_with("fwd") ||
+                    queryName.ends_with("rev")) {
+                    HandleName(queryName, RecordType::CCS);
+                } else {
+                    // CCS segment
+                    assert(std::isdigit(queryName.back()));
+                    HandleName(queryName, RecordType::SEGMENT);
+                }
             } else {
+                // all other read types
                 HandleName(queryName, RecordType::UNKNOWN);
             }
         }
@@ -391,6 +401,13 @@ public:
 
     std::vector<int32_t> CandidateRgIds(const std::string& movieName, const RecordType type)
     {
+        if (type == RecordType::SEGMENT) {
+            return {
+                ReadGroupInfo::IdToInt(MakeReadGroupId(movieName, "SEGMENT")),
+                ReadGroupInfo::IdToInt(MakeLegacyReadGroupId(movieName, "SEGMENT")),
+            };
+        }
+
         if (type == RecordType::CCS) {
             return {
                 ReadGroupInfo::IdToInt(MakeReadGroupId(movieName, "CCS")),
@@ -421,7 +438,7 @@ public:
     void HandleName(const std::string& queryName, const RecordType type)
     {
         const auto nameParts = Split(queryName, '/');
-        if (nameParts.size() < 2) {
+        if (std::ssize(nameParts) < 2) {
             throw std::runtime_error{"[pbbam] PBI filter ERROR: requested QNAME (" + queryName +
                                      ") is not a valid PacBio BAM QNAME. See spec for details"};
         }
@@ -440,13 +457,13 @@ public:
                     ") is not a valid PacBio BAM QNAME. ZMW id must be a number."};
             }
         }();
-
         if (IsCcsOrTranscript(type)) {
+            // normal CCS and TRANSCRIPT types have no interval
             zmw->emplace(zmwId, std::optional<QueryIntervals>{});
         } else {
 
-            const auto queryIntervalParts = Split(nameParts.at(2), '_');
-            if (queryIntervalParts.size() != 2) {
+            const auto queryIntervalParts = Split(nameParts.back(), '_');
+            if (std::ssize(queryIntervalParts) != 2) {
                 throw std::runtime_error{"[pbbam] PBI filter ERROR: requested QNAME (" + queryName +
                                          ") is not a valid PacBio BAM QNAME. See spec for details"};
             }
@@ -504,19 +521,16 @@ private:
 PbiQueryNameFilter::PbiQueryNameFilter(const std::string& qname, const Compare::Type cmp)
     : d_{std::make_unique<PbiQueryNameFilter::PbiQueryNameFilterPrivate>(
           std::vector<std::string>{1, qname}, cmp)}
-{
-}
+{}
 
 PbiQueryNameFilter::PbiQueryNameFilter(const std::vector<std::string>& queryNames,
                                        const Compare::Type cmp)
     : d_{std::make_unique<PbiQueryNameFilter::PbiQueryNameFilterPrivate>(queryNames, cmp)}
-{
-}
+{}
 
 PbiQueryNameFilter::PbiQueryNameFilter(const PbiQueryNameFilter& other)
     : d_{std::make_unique<PbiQueryNameFilter::PbiQueryNameFilterPrivate>(other.d_)}
-{
-}
+{}
 
 PbiQueryNameFilter::PbiQueryNameFilter(PbiQueryNameFilter&&) noexcept = default;
 
@@ -542,8 +556,7 @@ PbiReadGroupFilter::PbiReadGroupFilter(const std::vector<int32_t>& rgIds, const
 
 PbiReadGroupFilter::PbiReadGroupFilter(const int32_t rgId, const Compare::Type cmp)
     : PbiReadGroupFilter{std::vector<int32_t>{rgId}, cmp}
-{
-}
+{}
 
 PbiReadGroupFilter::PbiReadGroupFilter(const std::vector<ReadGroupInfo>& readGroups,
                                        const Compare::Type cmp)
@@ -554,8 +567,7 @@ PbiReadGroupFilter::PbiReadGroupFilter(const std::vector<ReadGroupInfo>& readGro
 
 PbiReadGroupFilter::PbiReadGroupFilter(const ReadGroupInfo& rg, const Compare::Type cmp)
     : PbiReadGroupFilter{std::vector<ReadGroupInfo>{rg}, cmp}
-{
-}
+{}
 
 PbiReadGroupFilter::PbiReadGroupFilter(const std::vector<std::string>& rgIds,
                                        const Compare::Type cmp)
@@ -570,8 +582,7 @@ PbiReadGroupFilter::PbiReadGroupFilter(const std::vector<std::string>& rgIds,
 PbiReadGroupFilter::PbiReadGroupFilter(const std::string& rgId,
                                        const Compare::Type cmp)  //: cmp_{cmp}
     : PbiReadGroupFilter{std::vector<std::string>{rgId}, cmp}
-{
-}
+{}
 
 bool PbiReadGroupFilter::Accepts(const PbiRawData& idx, const size_t row) const
 {
@@ -731,8 +742,7 @@ void PbiReferenceNameFilter::Validate() const
 // PbiZmwFilter
 
 PbiZmwFilter::PbiZmwFilter(const int32_t zmw, const Compare::Type cmp) : cmp_{cmp}, singleZmw_{zmw}
-{
-}
+{}
 
 PbiZmwFilter::PbiZmwFilter(std::vector<int32_t> whitelist, const Compare::Type cmp)
     : cmp_{cmp}, zmwLookup_{whitelist.begin(), whitelist.end()}
