@@ -3,20 +3,19 @@
 
 #include <pbbam/Config.h>
 
+#include <pbbam/DataSet.h>
+
 #include <deque>
+#include <optional>
 #include <string>
 #include <utility>
 
-#include <boost/optional.hpp>
-
-#include <pbbam/DataSet.h>
-
 namespace PacBio {
 namespace BAM {
 
 using StitchingSources = std::deque<std::pair<std::string, std::string>>;
 
-inline boost::optional<std::string> ScrapsFileId(const ExternalResource& resource)
+inline std::optional<std::string> ScrapsFileId(const ExternalResource& resource)
 {
     const auto& childResources = resource.ExternalResources();
     for (const auto& childResource : childResources) {
@@ -26,7 +25,7 @@ inline boost::optional<std::string> ScrapsFileId(const ExternalResource& resourc
             return childResource.ResourceId();
         }
     }
-    return boost::none;
+    return {};
 }
 
 inline StitchingSources SourcesFromDataset(const DataSet& dataset)
@@ -36,8 +35,8 @@ inline StitchingSources SourcesFromDataset(const DataSet& dataset)
     const ExternalResources& resources = dataset.ExternalResources();
     for (const ExternalResource& resource : resources) {
 
-        boost::optional<std::string> primaryId;
-        boost::optional<std::string> scrapsId;
+        std::optional<std::string> primaryId;
+        std::optional<std::string> scrapsId;
 
         // if resource is possible "primary" BAM, store & look for associated scraps
         const auto& metatype = resource.MetaType();
@@ -49,8 +48,8 @@ inline StitchingSources SourcesFromDataset(const DataSet& dataset)
 
         // if found, resolve paths & store
         if (primaryId && scrapsId) {
-            std::string primaryFn = dataset.ResolvePath(primaryId.get());
-            std::string scrapsFn = dataset.ResolvePath(scrapsId.get());
+            std::string primaryFn = dataset.ResolvePath(*primaryId);
+            std::string scrapsFn = dataset.ResolvePath(*scrapsId);
             sources.emplace_back(std::make_pair(primaryFn, scrapsFn));
         }
     }
