@@ -5,11 +5,11 @@
 
 #include "ZmwChunkedFastxReaderImpl.h"
 
-#include <memory>
+#include <pbbam/Deleters.h>
 
 #include <htslib/kseq.h>
 
-#include <pbbam/Deleters.h>
+#include <memory>
 
 namespace PacBio {
 namespace BAM {
@@ -17,7 +17,7 @@ namespace BAM {
 class ZmwChunkedFastxBgzfReader final : public ZmwChunkedFastxReaderImpl
 {
 public:
-    ZmwChunkedFastxBgzfReader(std::string filename, const size_t numChunks);
+    ZmwChunkedFastxBgzfReader(std::string filename, size_t numChunks);
 
     void Seek(uint64_t pos) final;
     FastaSequence ReadNextFasta(bool skipName) final;
@@ -32,7 +32,9 @@ private:
     {
         void operator()(kseq_t* seq) const noexcept
         {
-            if (seq) kseq_destroy(seq);
+            if (seq) {
+                kseq_destroy(seq);
+            }
             seq = nullptr;
         }
     };
