@@ -5,12 +5,13 @@
 
 #include "ZmwChunkedFastxReaderImpl.h"
 
-#include <cstdio>
+#include <pbcopper/utility/Deleters.h>
+
+#include <htslib/kseq.h>
 
 #include <memory>
 
-#include <htslib/kseq.h>
-#include <pbcopper/utility/Deleters.h>
+#include <cstdio>
 
 namespace PacBio {
 namespace BAM {
@@ -18,7 +19,7 @@ namespace BAM {
 class ZmwChunkedFastxTextReader final : public ZmwChunkedFastxReaderImpl
 {
 public:
-    ZmwChunkedFastxTextReader(std::string filename, const size_t numChunks);
+    ZmwChunkedFastxTextReader(std::string filename, size_t numChunks);
 
     void Seek(uint64_t pos) final;
     FastaSequence ReadNextFasta(bool skipName) final;
@@ -38,7 +39,9 @@ private:
     {
         void operator()(kseq_t* seq) const noexcept
         {
-            if (seq) kseq_destroy(seq);
+            if (seq) {
+                kseq_destroy(seq);
+            }
             seq = nullptr;
         }
     };
