@@ -1,5 +1,3 @@
-// Author: Derek Barnett
-
 #include <pbbam/BgzipFastqWriter.h>
 
 #include <string>
@@ -26,8 +24,9 @@ TEST(BAM_BgzipFastqWriter, writes_bgzf_fastq_data_to_file)
 
     {
         BgzipFastqWriter writer{fn};
-        for (const auto& seq : sequences)
+        for (const auto& seq : sequences) {
             writer.Write(seq);
+        }
     }
     EXPECT_EQ(HtslibCompression::BGZIP, FormatUtils::CompressionType(fn));
 
